==> symbol-processing-aa-embeddable-2.1.0-1.0.29-sources.jar: org/jetbrains/kotlin/ir/backend/js/checkers/expressions/JsKlibJsCodeCallChecker.kt <==
/*
 * Copyright 2010-2023 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.ir.backend.js.checkers.expressions

import ksp.com.google.gwt.dev.js.parserExceptions.AbortParsingException
import ksp.com.google.gwt.dev.js.rhino.CodePosition
import ksp.com.google.gwt.dev.js.rhino.ErrorReporter
import ksp.org.jetbrains.kotlin.config.languageVersionSettings
import ksp.org.jetbrains.kotlin.ir.IrDiagnosticReporter
import ksp.org.jetbrains.kotlin.ir.backend.js.checkers.JsKlibCallChecker
import ksp.org.jetbrains.kotlin.ir.backend.js.checkers.JsKlibDiagnosticContext
import ksp.org.jetbrains.kotlin.ir.backend.js.checkers.JsKlibErrors
import ksp.org.jetbrains.kotlin.ir.backend.js.checkers.at
import ksp.org.jetbrains.kotlin.ir.expressions.IrCall
import ksp.org.jetbrains.kotlin.ir.expressions.IrConst
import ksp.org.jetbrains.kotlin.ir.expressions.IrConstKind
import ksp.org.jetbrains.kotlin.ir.expressions.IrExpression
import ksp.org.jetbrains.kotlin.ir.util.fqNameWhenAvailable
import ksp.org.jetbrains.kotlin.js.backend.ast.JsFunctionScope
import ksp.org.jetbrains.kotlin.js.backend.ast.JsProgram
import ksp.org.jetbrains.kotlin.js.backend.ast.JsRootScope
import ksp.org.jetbrains.kotlin.js.parser.parseExpressionOrStatement
import ksp.org.jetbrains.kotlin.name.JsStandardClassIds

object JsKlibJsCodeCallChecker : JsKlibCallChecker {
    private val jsCodeFqName = JsStandardClassIds.Callables.JsCode.asSingleFqName()

    override fun check(expression: IrCall, context: JsKlibDiagnosticContext, reporter: IrDiagnosticReporter) {
        // Do not check IR from K1, because there are corresponding K1 FE checks in JsCallChecker
        if (!context.compilerConfiguration.languageVersionSettings.languageVersion.usesK2) {
            return
        }
        if (expression.symbol.owner.fqNameWhenAvailable != jsCodeFqName) {
            return
        }

        val jsCodeExpr = expression.getValueArgument(0)
        // K2 frontend checker FirJsCodeConstantArgumentChecker checks that the passing argument is a constant.
        // IrConstOnlyNecessaryTransformer must evaluate an argument expression and fold it into a string constant.
        if (jsCodeExpr !is IrConst || jsCodeExpr.kind != IrConstKind.String) {
            // Do not ignore the error if the argument does not fit the expectations.
            // The IR can be generated in the plugin, avoiding FE checks and IrConstOnlyNecessaryTransformer.
            // Bugs are also possible in IrConstOnlyNecessaryTransformer.
            reporter.at(jsCodeExpr ?: expression, context).report(JsKlibErrors.JSCODE_CAN_NOT_VERIFY_JAVASCRIPT)
            return
        }

        val jsCodeStr = jsCodeExpr.value as String

        try {
            val parserScope = JsFunctionScope(JsRootScope(JsProgram()), "<js fun>")
            val fileName = context.containingFile?.fileEntry?.name ?: "<unknown file>"
            val jsErrorReporter = JsErrorReporter(jsCodeExpr, context, reporter)
            val statements = parseExpressionOrStatement(jsCodeStr, jsErrorReporter, parserScope, CodePosition(0, 0), fileName)
            if (statements.isNullOrEmpty()) {
                reporter.at(jsCodeExpr, context).report(JsKlibErrors.JSCODE_NO_JAVASCRIPT_PRODUCED)
            }
        } catch (e: AbortParsingException) {
            // ignore
        }
    }

    private class JsErrorReporter(
        val codeExpression: IrExpression,
        val context: JsKlibDiagnosticContext,
        val reporter: IrDiagnosticReporter,
    ) : ErrorReporter {
        override fun warning(message: String, startPosition: CodePosition, endPosition: CodePosition) {
            reporter.at(codeExpression, context).report(JsKlibErrors.JSCODE_WARNING, message)
        }

        override fun error(message: String, startPosition: CodePosition, endPosition: CodePosition) {
            reporter.at(codeExpression, context).report(JsKlibErrors.JSCODE_ERROR, message)
            throw AbortParsingException()
        }
    }
}

==> symbol-processing-aa-embeddable-2.3.3-sources.jar: org/jetbrains/kotlin/ir/backend/js/checkers/expressions/JsKlibJsCodeCallChecker.kt <==
/*
 * Copyright 2010-2023 JetBrains s.r.o. and Kotlin Programming Language contributors.
 * Use of this source code is governed by the Apache 2.0 license that can be found in the license/LICENSE.txt file.
 */

package ksp.org.jetbrains.kotlin.ir.backend.js.checkers.expressions

import ksp.com.google.gwt.dev.js.parserExceptions.AbortParsingException
import ksp.com.google.gwt.dev.js.rhino.CodePosition
import ksp.com.google.gwt.dev.js.rhino.ErrorReporter
import ksp.org.jetbrains.kotlin.config.languageVersionSettings
import ksp.org.jetbrains.kotlin.ir.IrDiagnosticReporter
import ksp.org.jetbrains.kotlin.ir.backend.js.checkers.JsKlibCallChecker
import ksp.org.jetbrains.kotlin.ir.backend.js.checkers.JsKlibDiagnosticContext
import ksp.org.jetbrains.kotlin.ir.backend.js.checkers.JsKlibErrors
import ksp.org.jetbrains.kotlin.ir.backend.js.checkers.at
import ksp.org.jetbrains.kotlin.ir.expressions.IrCall
import ksp.org.jetbrains.kotlin.ir.expressions.IrConst
import ksp.org.jetbrains.kotlin.ir.expressions.IrConstKind
import ksp.org.jetbrains.kotlin.ir.expressions.IrExpression
import ksp.org.jetbrains.kotlin.ir.util.hasEqualFqName
import ksp.org.jetbrains.kotlin.js.backend.ast.JsFunctionScope
import ksp.org.jetbrains.kotlin.js.backend.ast.JsProgram
import ksp.org.jetbrains.kotlin.js.backend.ast.JsRootScope
import ksp.org.jetbrains.kotlin.js.parser.parseExpressionOrStatement
import ksp.org.jetbrains.kotlin.name.JsStandardClassIds

object JsKlibJsCodeCallChecker : JsKlibCallChecker {
    private val jsCodeFqName = JsStandardClassIds.Callables.JsCode.asSingleFqName()

    private fun IrCall.isJsFunCall(): Boolean {
        return if (this.symbol.isBound) {
            symbol.owner.hasEqualFqName(jsCodeFqName)
        } else {
            symbol.hasEqualFqName(jsCodeFqName)
        }
    }

    override fun check(expression: IrCall, context: JsKlibDiagnosticContext, reporter: IrDiagnosticReporter) {
        // Do not check IR from K1, because there are corresponding K1 FE checks in JsCallChecker
        if (!context.compilerConfiguration.languageVersionSettings.languageVersion.usesK2) {
            return
        }

        if (!expression.isJsFunCall()) {
            return
        }

        val jsCodeExpr = expression.arguments[0]
        // K2 frontend checker FirJsCodeConstantArgumentChecker checks that the passing argument is a constant.
        // IrConstOnlyNecessaryTransformer must evaluate an argument expression and fold it into a string constant.
        if (jsCodeExpr !is IrConst || jsCodeExpr.kind != IrConstKind.String) {
            // Do not ignore the error if the argument does not fit the expectations.
            // The IR can be generated in the plugin, avoiding FE checks and IrConstOnlyNecessaryTransformer.
            // Bugs are also possible in IrConstOnlyNecessaryTransformer.
            reporter.at(jsCodeExpr ?: expression, context).report(JsKlibErrors.JSCODE_CAN_NOT_VERIFY_JAVASCRIPT)
            return
        }

        val jsCodeStr = jsCodeExpr.value as String

        try {
            val parserScope = JsFunctionScope(JsRootScope(JsProgram()), "<js fun>")
            val fileName = context.containingFile?.fileEntry?.name ?: "<unknown file>"
            val jsErrorReporter = JsErrorReporter(jsCodeExpr, context, reporter)
            val statements = parseExpressionOrStatement(jsCodeStr, jsErrorReporter, parserScope, CodePosition(0, 0), fileName)
            if (statements.isNullOrEmpty()) {
                reporter.at(jsCodeExpr, context).report(JsKlibErrors.JSCODE_NO_JAVASCRIPT_PRODUCED)
            }
        } catch (e: AbortParsingException) {
            // ignore
        }
    }

    private class JsErrorReporter(
        val codeExpression: IrExpression,
        val context: JsKlibDiagnosticContext,
        val reporter: IrDiagnosticReporter,
    ) : ErrorReporter {
        override fun warning(message: String, startPosition: CodePosition, endPosition: CodePosition) {
            reporter.at(codeExpression, context).report(JsKlibErrors.JSCODE_WARNING, message)
        }

        override fun error(message: String, startPosition: CodePosition, endPosition: CodePosition) {
            reporter.at(codeExpression, context).report(JsKlibErrors.JSCODE_ERROR, message)
            throw AbortParsingException()
        }
    }
}
